SystemOrganization addCategory: #DefragFAT!

TestCase subclass: #DFDefragAppOneFileTest
	instanceVariableNames: 'defrag app file'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DefragFAT'!

!DFDefragAppOneFileTest methodsFor: 'as yet unclassified' stamp: 'hjo 4/18/2011 00:14'!
setUp

	|pc|
	pc := DFIBMPC new.
	pc doInstallMemorySize: 3.
	pc doInstallHardDiskSize: 40.
	pc hardDisk doFormatFAT.
	file := pc hardDisk fileSystem doAddFile.
	file doAddContinuousByesFromIndex: 20 withBytes: 'Hel'.
	file doAddContinuousByesFromIndex: 3 withBytes: 'lo W'.
	file doAddContinuousByesFromIndex: 10 withBytes: 'orld!!!!'.
	app := DFDefragApp new.
	app pc: pc.
! !

!DFDefragAppOneFileTest methodsFor: 'as yet unclassified' stamp: 'hjo 4/18/2011 00:19'!
testDefrag

	self assert: 3 equals: file fragCount.
	app doDefrag.
	self assert: 1 equals: file fragCount.! !

!DFDefragAppOneFileTest methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 15:20'!
testSetUp

	! !

TestCase subclass: #DFDiskContinuousBytesTest
	instanceVariableNames: 'diskContinuousBytes hardDisk fat'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DefragFAT'!

!DFDiskContinuousBytesTest methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 17:16'!
setUp

	super setUp.
	hardDisk := DFHardDisk new.
	hardDisk doAllocateSize: 30.
	fat := DFFAT new.
	fat hardDisk: hardDisk.
	diskContinuousBytes := DFDiskContinuousBytes new.
	diskContinuousBytes fileSystem: fat.! !

!DFDiskContinuousBytesTest methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 17:57'!
testFATAccessBytes

	diskContinuousBytes startingIndex: 10.
	diskContinuousBytes doWriteBytes: (ByteArray withAll: 'hello world, baby').
	! !

!DFDiskContinuousBytesTest methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 17:58'!
testFATAccessBytesErrorIfTooLong

	diskContinuousBytes startingIndex: 25.
	self should: [diskContinuousBytes doWriteBytes: (ByteArray withAll: 'hello world, baby')] raise: Error  
	.
	! !

!DFDiskContinuousBytesTest methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 16:08'!
testFATAccessErrorIfStartingIndexTooBig

	self should: [ diskContinuousBytes startingIndex: 40] raise: Error.
	  
	! !

!DFDiskContinuousBytesTest methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 15:57'!
testFATAccessErrorIfStartingIndexTooSmall

	self should: [ diskContinuousBytes startingIndex: -3] raise: Error.
	  
	! !

!DFDiskContinuousBytesTest methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 15:58'!
testFATAccessGoodStartingIndex

	self shouldnt: [ diskContinuousBytes startingIndex: 1 ] raise: Error.
	  
	! !

!DFDiskContinuousBytesTest methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 20:37'!
testRead

	| data |
	data := ByteArray withAll: 'hello world, baby'.
	diskContinuousBytes startingIndex: 10.
	diskContinuousBytes doWriteBytes: data.
	self assert: data equals: diskContinuousBytes bytes! !

!DFDiskContinuousBytesTest methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 20:45'!
testWrite

	| before after getBytes getFront getMiddle getBack |
	getBytes := [hardDisk diskBytes copy].
	before := getBytes value.
	diskContinuousBytes startingIndex: 10.
	diskContinuousBytes doWriteBytes: (ByteArray withAll: 'hello world, baby').
	after := getBytes value.
	
	getFront := [:bytes| bytes copyFrom:1 to:9 ].
	getMiddle := [:bytes| bytes copyFrom:10 to: 26 ].
	getBack := [:bytes| bytes copyFrom:27 to: bytes size ].
	self assert: (getFront value: before) = (getFront value: after).
	self deny:  (getMiddle value: before) = (getMiddle value: after).
	self assert: 'hello world, baby' equals: (getMiddle value: after) asString.
	self assert: (getBack value: before) = (getBack value: after).
! !

TestCase subclass: #DFFATTest
	instanceVariableNames: 'fat hardDisk'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DefragFAT'!

!DFFATTest methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 21:39'!
setUp

	super setUp.
	hardDisk := DFHardDisk new.
	hardDisk doAllocateSize: 50.
	hardDisk doFormatFAT.
	fat := hardDisk fileSystem.  ! !

!DFFATTest methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 23:33'!
testDoAddFile

	fat doAddFile.
	self assert: 1 equals: fat files size! !

!DFFATTest methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 21:41'!
testEmpty

	|bytes|
	bytes := ByteArray withAll: #(3 5 10).
	self deny: (fat canPut: bytes startingAt: 0).
	self assert: (fat canPut: bytes startingAt: 25).
	self assert: (fat canPut: bytes startingAt: 48).
	self deny: (fat canPut: bytes startingAt: 49).! !

!DFFATTest methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 23:23'!
testOneAlloc

	|file bytes|
	file := fat doAddFile.
	file doAddContinuousByesFromIndex: 15 withBytes: (ByteArray withAll: 'i am an headache for you'). 
	bytes := ByteArray withAll: #(3 5 10).
	self deny: (fat canPut: bytes startingAt: 0).
	self assert: (fat canPut: bytes startingAt: 12).
	self deny: (fat canPut: bytes startingAt: 13).
	self deny: (fat canPut: bytes startingAt: 30).
	self deny: (fat canPut: bytes startingAt: 38).
	self assert: (fat canPut: bytes startingAt: 39).
	self assert: (fat canPut: bytes startingAt: 48).
	self deny: (fat canPut: bytes startingAt: 49).
	! !

Object subclass: #DFDefragApp
	instanceVariableNames: 'pc'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DefragFAT'!

!DFDefragApp methodsFor: 'as yet unclassified' stamp: 'hjo 4/18/2011 00:12'!
doDefrag

	pc hardDisk fileSystem files do: [:each| self doDefragFor: each]! !

!DFDefragApp methodsFor: 'as yet unclassified' stamp: 'hjo 4/18/2011 00:23'!
doDefragFor: aDFFile 

	| contents |
	pc memory reset.
	contents := ByteArray streamContents: [:stream| aDFFile diskContinuousBytes do: [:each| stream nextPutAll: each bytes ] ].
	aDFFile doDeleteFile.
	self flag: 'starting index is 1 just for now.'.
	aDFFile doAddContinuousByesFromIndex: 1 withBytes:  contents! !

!DFDefragApp methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 15:32'!
pc: aDFIBMPC 

	pc := aDFIBMPC! !

Object subclass: #DFDiskContinuousBytes
	instanceVariableNames: 'startingIndex length fileSystem'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DefragFAT'!

!DFDiskContinuousBytes methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 23:47'!
allocationCollidesToPut: aByteArray startingAt: anInteger 

	^((anInteger + aByteArray size - 1) < startingIndex or: 
	[
		(startingIndex + length - 1)  < anInteger
	]) not.
! !

!DFDiskContinuousBytes methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 19:45'!
bytes

	^fileSystem bytesFrom: startingIndex to: startingIndex + length - 1! !

!DFDiskContinuousBytes methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 19:27'!
doWriteBytes: aByteArray 

	fileSystem doWriteBytes: aByteArray withStartingIndex: startingIndex.	
	length := aByteArray size.! !

!DFDiskContinuousBytes methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 17:34'!
fileSystem: aFileSystem 

	fileSystem := aFileSystem! !

!DFDiskContinuousBytes methodsFor: 'as yet unclassified' stamp: 'hjo 4/18/2011 00:10'!
startingIndex

	^ startingIndex! !

!DFDiskContinuousBytes methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 17:35'!
startingIndex: anInteger 

	anInteger > fileSystem hardDiskSize ifTrue: [self error: 'starting index greater than harddisk limit'].  	
	anInteger < 1 ifTrue: [self error: 'starting index less than 1'].  	
	startingIndex := anInteger ! !

Object subclass: #DFFAT
	instanceVariableNames: 'files hardDisk'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DefragFAT'!

!DFFAT methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 21:06'!
bytesFrom: integer1 to: integer2 

	^hardDisk bytesFrom: integer1 to: integer2 ! !

!DFFAT methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 23:34'!
canPut: aByteArray startingAt: anInteger 

	(anInteger >= 1 and: [anInteger + aByteArray size - 1 <= hardDisk diskSize])
		ifFalse: [^false].  
	^files allSatisfy: [:f| (f allocationCollidesToPut: aByteArray startingAt: anInteger) not ]! !

!DFFAT methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 23:17'!
doAddFile

	| newFile |
	newFile := DFFile new.
	newFile fileSystem: self.
	files add: newFile.
	^newFile! !

!DFFAT methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 21:06'!
doWriteBytes: aByteString withStartingIndex: anInteger 

	hardDisk doWriteBytes: aByteString withStartingIndex: anInteger! !

!DFFAT methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 17:02'!
files

	^ files! !

!DFFAT methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 15:59'!
hardDisk: aDFHardDisk 

	hardDisk := aDFHardDisk ! !

!DFFAT methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 16:19'!
hardDiskSize

	^hardDisk diskSize! !

!DFFAT methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 17:07'!
initialize 

	super initialize.
	files := OrderedCollection new.! !

Object subclass: #DFFile
	instanceVariableNames: 'continuousBytes fileSystem'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DefragFAT'!

!DFFile methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 23:25'!
allocationCollidesToPut: aByteArray startingAt: anInteger 

	^continuousBytes anySatisfy: [:each| each allocationCollidesToPut: aByteArray startingAt: anInteger ]! !

!DFFile methodsFor: 'as yet unclassified' stamp: 'hjo 4/18/2011 00:04'!
diskContinuousBytes

	^continuousBytes! !

!DFFile methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 21:05'!
doAddContinuousByesFromIndex: anInteger withBytes: aByteArray 

	| bytes |
	bytes := DFDiskContinuousBytes new.
	bytes fileSystem: fileSystem.
	bytes startingIndex: anInteger.
	bytes doWriteBytes: aByteArray.
	
	continuousBytes add: bytes.! !

!DFFile methodsFor: 'as yet unclassified' stamp: 'hjo 4/18/2011 00:21'!
doDeleteFile

	continuousBytes := OrderedCollection new.! !

!DFFile methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 17:19'!
fileSystem: aDFFAT 

	fileSystem := aDFFAT! !

!DFFile methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 20:53'!
fragCount

	^continuousBytes size! !

!DFFile methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 20:53'!
initialize 

	super initialize.
	continuousBytes := OrderedCollection new.! !

Object subclass: #DFHardDisk
	instanceVariableNames: 'diskBytes fileSystem'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DefragFAT'!

!DFHardDisk methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 21:04'!
bytesFrom: integer1 to: integer2 

	^diskBytes copyFrom: integer1 to: integer2! !

!DFHardDisk methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 19:37'!
diskBytes

	^ diskBytes! !

!DFHardDisk methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 16:20'!
diskSize

	^diskBytes size! !

!DFHardDisk methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 21:04'!
doAllocateSize: anInteger 

	self assert: diskBytes isNil.
	diskBytes := ByteArray new: anInteger.! !

!DFHardDisk methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 17:14'!
doFormatFAT

	fileSystem := DFFAT new.
	fileSystem hardDisk: self.
! !

!DFHardDisk methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 21:05'!
doWriteBytes: aString withStartingIndex: anInteger 

	diskBytes replaceFrom: anInteger to: anInteger + aString size - 1 with: aString! !

!DFHardDisk methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 17:12'!
fileSystem

	^ fileSystem! !

!DFHardDisk methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 17:07'!
initialize 

	super initialize.! !

Object subclass: #DFIBMPC
	instanceVariableNames: 'memory disk'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DefragFAT'!

!DFIBMPC methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 17:42'!
doInstallHardDiskSize: anInteger

	self assert: disk isNil.
	disk := DFHardDisk new.
	disk doAllocateSize: anInteger.  ! !

!DFIBMPC methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 17:43'!
doInstallMemorySize: anInteger 

	self assert: memory isNil.
	memory := DFMemory new.
	memory doInstallSize: anInteger! !

!DFIBMPC methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 15:22'!
hardDisk

	^disk! !

!DFIBMPC methodsFor: 'as yet unclassified' stamp: 'hjo 4/18/2011 00:02'!
memory

	^ memory! !

Object subclass: #DFMemory
	instanceVariableNames: 'byteArray'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'DefragFAT'!

!DFMemory methodsFor: 'as yet unclassified' stamp: 'hjo 4/17/2011 17:40'!
doInstallSize: anInteger
	 
	byteArray := ByteArray new: anInteger! !

!DFMemory methodsFor: 'as yet unclassified' stamp: 'hjo 4/18/2011 00:03'!
reset

	self flag: 'i dont know what to here yet'! !
